@@ -158,7 +158,7 @@ class KenwoodLiveRadio(chirp_common.LiveRadio):
             return self.__memcache[number]
 
         result = command(self.pipe, *self._cmd_get_memory(number))
-        if result == "N":
+        if result == "N" or result == "E":
             mem = chirp_common.Memory()
             mem.number = number
             mem.empty = True
@@ -268,8 +268,23 @@ TH_D7_SETTINGS = {
               "9 digit NMEA", "6 digit Magellan", "DGPS"],
 }
 
+class KenwoodOldLiveRadio(KenwoodLiveRadio):
+    _kenwood_valid_tones = list(chirp_common.OLD_TONES)
+
+    def set_memory(self, memory):
+        supported_tones = list(chirp_common.OLD_TONES)
+        supported_tones.remove(69.3)
+        if memory.rtone not in supported_tones:
+            raise errors.UnsupportedToneError("This radio does not support " +
+                                              "tone %.1fHz" % memory.rtone)
+        if memory.ctone not in supported_tones:
+            raise errors.UnsupportedToneError("This radio does not support " +
+                                              "tone %.1fHz" % memory.ctone)
+
+        return KenwoodLiveRadio.set_memory(self, memory)
+
 @directory.register
-class THD7Radio(KenwoodLiveRadio):
+class THD7Radio(KenwoodOldLiveRadio):
     """Kenwood TH-D7"""
     MODEL = "TH-D7"
 
@@ -475,7 +490,7 @@ class THD7GRadio(THD7Radio):
     MODEL = "TH-D7G"
 
 @directory.register
-class TMD700Radio(KenwoodLiveRadio):
+class TMD700Radio(KenwoodOldLiveRadio):
     """Kenwood TH-D700"""
     MODEL = "TM-D700"
 
@@ -542,40 +557,13 @@ class TMD700Radio(KenwoodLiveRadio):
 
         return mem
 
-OLD_TONES = list(chirp_common.TONES)
-OLD_TONES.remove(159.8)
-OLD_TONES.remove(165.5)
-OLD_TONES.remove(171.3)
-OLD_TONES.remove(177.3)
-OLD_TONES.remove(183.5)
-OLD_TONES.remove(189.9)
-OLD_TONES.remove(196.6)
-OLD_TONES.remove(199.5)
-OLD_TONES.remove(206.5)
-OLD_TONES.remove(229.1)
-OLD_TONES.remove(254.1)
-
 @directory.register
-class TMV7Radio(KenwoodLiveRadio):
+class TMV7Radio(KenwoodOldLiveRadio):
     """Kenwood TM-V7"""
     MODEL = "TM-V7"
 
     mem_upper_limit = 200 # Will be updated
 
-    _kenwood_valid_tones = list(OLD_TONES)
-
-    def set_memory(self, memory):
-        supported_tones = list(OLD_TONES)
-        supported_tones.remove(69.3)
-        if memory.rtone not in supported_tones:
-            raise errors.UnsupportedToneError("This radio does not support " +
-                                              "tone %.1fHz" % memory.rtone)
-        if memory.ctone not in supported_tones:
-            raise errors.UnsupportedToneError("This radio does not support " +
-                                              "tone %.1fHz" % memory.ctone)
-
-        return KenwoodLiveRadio.set_memory(self, memory)
-
     def get_features(self):
         rf = chirp_common.RadioFeatures()
         rf.has_dtcs = False
@@ -680,10 +668,63 @@ class TMG707Radio(TMV7Radio):
         rf = TMV7Radio.get_features(self)
         rf.has_sub_devices = False
         rf.memory_bounds = (1, 180)
-        rf.valid_bands = [(144000000, 148000000),
-                          (430000000, 450000000)]
+        rf.valid_bands = [(118000000, 174000000),
+                          (300000000, 520000000),
+                          (800000000, 999000000)]
         return rf
 
+THG71_STEPS = [ 5, 6.25, 10, 12.5, 15, 20, 25, 30, 50, 100 ]
+@directory.register
+class THG71Radio(TMV7Radio):
+    """Kenwood TH-G71"""
+    MODEL = "TH-G71"
+
+    def get_features(self):
+        rf = TMV7Radio.get_features(self)
+        rf.has_tuning_step = True
+        rf.valid_tuning_steps = list(THG71_STEPS)
+        rf.valid_name_length = 6
+        rf.has_sub_devices = False
+        rf.valid_bands = [(118000000, 174000000),
+                          (320000000, 470000000),
+						  (800000000, 945000000)]		
+        return rf
+
+    def _make_mem_spec(self, mem):
+        spec = ( \
+            "%011i" % mem.freq,
+            "%X" % THG71_STEPS.index(mem.tuning_step),
+            "%i" % util.get_dict_rev(DUPLEX, mem.duplex),
+            "0",
+            "%i" % (mem.tmode == "Tone"),
+            "%i" % (mem.tmode == "TSQL"),
+            "0",
+            "%02i" % (self._kenwood_valid_tones.index(mem.rtone) + 1),
+            "000",
+            "%02i" % (self._kenwood_valid_tones.index(mem.ctone) + 1),
+            "%09i" % mem.offset,
+            "%i" % ((mem.skip == "S") and 1 or 0))
+        return spec
+
+    def _parse_mem_spec(self, spec):
+        mem = chirp_common.Memory()
+        mem.number = int(spec[2])
+        mem.freq = int(spec[3])
+        mem.tuning_step = THG71_STEPS[int(spec[4], 16)]
+        mem.duplex = DUPLEX[int(spec[5])]
+        if int(spec[7]):
+            mem.tmode = "Tone"
+        elif int(spec[8]):
+            mem.tmode = "TSQL"
+        mem.rtone = self._kenwood_valid_tones[int(spec[10]) - 1]
+        mem.ctone = self._kenwood_valid_tones[int(spec[12]) - 1]
+        if spec[13]:
+            mem.offset = int(spec[13])
+        else:
+            mem.offset = 0
+        return mem
+
+		
 THF6A_STEPS = [5.0, 6.25, 8.33, 9.0, 10.0, 12.5, 15.0, 20.0, 25.0, 30.0, 50.0,
                100.0]
 
@@ -1084,6 +1125,12 @@ class TM271Radio(THK2Radio):
     def _cmd_set_memory_name(self, number, name):
         return "MN", "%03i,%s" % (number, name)
 
+@directory.register
+class TM281Radio(TM271Radio):
+    """Kenwood TM-281"""
+    MODEL = "TM-281"
+    # seems that this is a perfect clone of TM271 with just a different model
+
 def do_test():
     """Dev test"""
     mem = chirp_common.Memory()
@@ -1114,5 +1161,70 @@ def do_test():
     radio = tc(FakeSerial())
     radio.set_memory(mem)
 
+@directory.register
+class TM471Radio(THK2Radio):
+    """Kenwood TM-471"""
+    MODEL = "TM-471"
+    
+    def get_features(self):
+        rf = chirp_common.RadioFeatures()
+        rf.can_odd_split = False
+        rf.has_dtcs_polarity = False
+        rf.has_bank = False
+        rf.has_tuning_step = False
+        rf.valid_tmodes = ["", "Tone", "TSQL", "DTCS"]
+        rf.valid_modes = THK2_MODES
+        rf.valid_duplexes = THK2_DUPLEX
+        rf.valid_characters = THK2_CHARS
+        rf.valid_name_length = 6
+        rf.valid_bands = [(444000000, 479990000)]
+        rf.valid_skips = ["", "S"]
+        rf.valid_tuning_steps = [5.0]
+        rf.memory_bounds = (0, 99)
+        return rf
+
+    def _cmd_get_memory(self, number):
+        return "ME", "%03i" % number
+
+    def _cmd_get_memory_name(self, number):
+        return "MN", "%03i" % number
+
+    def _cmd_set_memory(self, number, spec):
+        return "ME", "%03i,%s" % (number, spec)
+
+    def _cmd_set_memory_name(self, number, name):
+        return "MN", "%03i,%s" % (number, name)
+
+def do_test():
+    """Dev test"""
+    mem = chirp_common.Memory()
+    mem.number = 1
+    mem.freq = 440000000
+    mem.duplex = "split"
+    mem.offset = 442000000
+
+    tc = THF6ARadio
+    class FakeSerial:
+        """Faked serial line"""
+        buf = ""
+        def write(self, buf):
+            """Write"""
+            self.buf = buf
+        def read(self, count):
+            """Read"""
+            if self.buf[:2] == "ID":
+                return "ID %s\r" % tc.MODEL
+            return self.buf
+        def setTimeout(self, foo):
+            """Set Timeout"""
+            pass
+        def setBaudrate(self, foo):
+            """Set Baudrate"""
+            pass
+
+    radio = tc(FakeSerial())
+    radio.set_memory(mem)
+
 if __name__ == "__main__":
     do_test()
+
